@@ -14,11 +14,10 @@ from django.contrib.admin.utils import (
 from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import models
 from django.utils import timezone
-from django.utils.encoding import force_text
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 
-class ListFilter(object):
+class ListFilter:
     title = None  # Human-readable title to appear in the right sidebar.
     template = 'admin/filter.html'
 
@@ -33,13 +32,13 @@ class ListFilter(object):
 
     def has_output(self):
         """
-        Returns True if some choices would be output for this filter.
+        Return True if some choices would be output for this filter.
         """
         raise NotImplementedError('subclasses of ListFilter must provide a has_output() method')
 
     def choices(self, changelist):
         """
-        Returns choices ready to be output in the template.
+        Return choices ready to be output in the template.
 
         `changelist` is the ChangeList to be displayed.
         """
@@ -47,13 +46,13 @@ class ListFilter(object):
 
     def queryset(self, request, queryset):
         """
-        Returns the filtered queryset.
+        Return the filtered queryset.
         """
         raise NotImplementedError('subclasses of ListFilter must provide a queryset() method')
 
     def expected_parameters(self):
         """
-        Returns the list of parameter names that are expected from the
+        Return the list of parameter names that are expected from the
         request's query string and that will be used by this filter.
         """
         raise NotImplementedError('subclasses of ListFilter must provide an expected_parameters() method')
@@ -64,8 +63,7 @@ class SimpleListFilter(ListFilter):
     parameter_name = None
 
     def __init__(self, request, params, model, model_admin):
-        super(SimpleListFilter, self).__init__(
-            request, params, model, model_admin)
+        super().__init__(request, params, model, model_admin)
         if self.parameter_name is None:
             raise ImproperlyConfigured(
                 "The list filter '%s' does not specify "
@@ -83,9 +81,9 @@ class SimpleListFilter(ListFilter):
 
     def value(self):
         """
-        Returns the value (in string format) provided in the request's
-        query string for this filter, if any. If the value wasn't provided then
-        returns None.
+        Return the value (in string format) provided in the request's
+        query string for this filter, if any, or None if the value wasn't
+        provided.
         """
         return self.used_parameters.get(self.parameter_name)
 
@@ -108,7 +106,7 @@ class SimpleListFilter(ListFilter):
         }
         for lookup, title in self.lookup_choices:
             yield {
-                'selected': self.value() == force_text(lookup),
+                'selected': self.value() == str(lookup),
                 'query_string': changelist.get_query_string({self.parameter_name: lookup}, []),
                 'display': title,
             }
@@ -122,8 +120,7 @@ class FieldListFilter(ListFilter):
         self.field = field
         self.field_path = field_path
         self.title = getattr(field, 'verbose_name', field_path)
-        super(FieldListFilter, self).__init__(
-            request, params, model, model_admin)
+        super().__init__(request, params, model, model_admin)
         for p in self.expected_parameters():
             if p in params:
                 value = params.pop(p)
@@ -135,7 +132,9 @@ class FieldListFilter(ListFilter):
     def queryset(self, request, queryset):
         try:
             return queryset.filter(**self.used_parameters)
-        except ValidationError as e:
+        except (ValueError, ValidationError) as e:
+            # Fields may raise a ValueError or ValidationError when converting
+            # the parameters to the correct type.
             raise IncorrectLookupParameters(e)
 
     @classmethod
@@ -165,8 +164,7 @@ class RelatedFieldListFilter(FieldListFilter):
         self.lookup_kwarg_isnull = '%s__isnull' % field_path
         self.lookup_val = request.GET.get(self.lookup_kwarg)
         self.lookup_val_isnull = request.GET.get(self.lookup_kwarg_isnull)
-        super(RelatedFieldListFilter, self).__init__(
-            field, request, params, model, model_admin, field_path)
+        super().__init__(field, request, params, model, model_admin, field_path)
         self.lookup_choices = self.field_choices(field, request, model_admin)
         if hasattr(field, 'verbose_name'):
             self.lookup_title = field.verbose_name
@@ -207,7 +205,7 @@ class RelatedFieldListFilter(FieldListFilter):
         }
         for pk_val, val in self.lookup_choices:
             yield {
-                'selected': self.lookup_val == force_text(pk_val),
+                'selected': self.lookup_val == str(pk_val),
                 'query_string': changelist.get_query_string({
                     self.lookup_kwarg: pk_val,
                 }, [self.lookup_kwarg_isnull]),
@@ -232,7 +230,7 @@ class BooleanFieldListFilter(FieldListFilter):
         self.lookup_kwarg2 = '%s__isnull' % field_path
         self.lookup_val = request.GET.get(self.lookup_kwarg)
         self.lookup_val2 = request.GET.get(self.lookup_kwarg2)
-        super(BooleanFieldListFilter, self).__init__(field, request, params, model, model_admin, field_path)
+        super().__init__(field, request, params, model, model_admin, field_path)
         if (self.used_parameters and self.lookup_kwarg in self.used_parameters and
                 self.used_parameters[self.lookup_kwarg] in ('1', '0')):
             self.used_parameters[self.lookup_kwarg] = bool(int(self.used_parameters[self.lookup_kwarg]))
@@ -274,8 +272,7 @@ class ChoicesFieldListFilter(FieldListFilter):
         self.lookup_kwarg_isnull = '%s__isnull' % field_path
         self.lookup_val = request.GET.get(self.lookup_kwarg)
         self.lookup_val_isnull = request.GET.get(self.lookup_kwarg_isnull)
-        super(ChoicesFieldListFilter, self).__init__(
-            field, request, params, model, model_admin, field_path)
+        super().__init__(field, request, params, model, model_admin, field_path)
 
     def expected_parameters(self):
         return [self.lookup_kwarg, self.lookup_kwarg_isnull]
@@ -294,7 +291,7 @@ class ChoicesFieldListFilter(FieldListFilter):
                 none_title = title
                 continue
             yield {
-                'selected': force_text(lookup) == self.lookup_val,
+                'selected': str(lookup) == self.lookup_val,
                 'query_string': changelist.get_query_string(
                     {self.lookup_kwarg: lookup}, [self.lookup_kwarg_isnull]
                 ),
@@ -362,8 +359,7 @@ class DateFieldListFilter(FieldListFilter):
                 (_('No date'), {self.field_generic + 'isnull': 'True'}),
                 (_('Has date'), {self.field_generic + 'isnull': 'False'}),
             )
-        super(DateFieldListFilter, self).__init__(
-            field, request, params, model, model_admin, field_path)
+        super().__init__(field, request, params, model, model_admin, field_path)
 
     def expected_parameters(self):
         params = [self.lookup_kwarg_since, self.lookup_kwarg_until]
@@ -404,8 +400,7 @@ class AllValuesFieldListFilter(FieldListFilter):
                                .distinct()
                                .order_by(field.name)
                                .values_list(field.name, flat=True))
-        super(AllValuesFieldListFilter, self).__init__(
-            field, request, params, model, model_admin, field_path)
+        super().__init__(field, request, params, model, model_admin, field_path)
 
     def expected_parameters(self):
         return [self.lookup_kwarg, self.lookup_kwarg_isnull]
@@ -421,7 +416,7 @@ class AllValuesFieldListFilter(FieldListFilter):
             if val is None:
                 include_none = True
                 continue
-            val = force_text(val)
+            val = str(val)
             yield {
                 'selected': self.lookup_val == val,
                 'query_string': changelist.get_query_string({
