@@ -15,7 +15,7 @@ from django.core.exceptions import (
 )
 from django.core.paginator import InvalidPage
 from django.db import models
-from django.db.models.expressions import F, OrderBy
+from django.db.models.expressions import Combinable, F, OrderBy
 from django.urls import reverse
 from django.utils.http import urlencode
 from django.utils.translation import gettext
@@ -289,7 +289,9 @@ class ChangeList:
             # the right column numbers absolutely, because there might be more
             # than one column associated with that ordering, so we guess.
             for field in ordering:
-                if isinstance(field, OrderBy):
+                if isinstance(field, (Combinable, OrderBy)):
+                    if not isinstance(field, OrderBy):
+                        field = field.asc()
                     if isinstance(field.expression, F):
                         order_type = 'desc' if field.descending else 'asc'
                         field = field.expression.name
