@@ -1,23 +1,20 @@
-from __future__ import unicode_literals
-
+import functools
 import gzip
 import os
 import re
 from difflib import SequenceMatcher
 
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured, ValidationError
-from django.utils import lru_cache
-from django.utils._os import upath
-from django.utils.encoding import force_text
+from django.core.exceptions import (
+    FieldDoesNotExist, ImproperlyConfigured, ValidationError,
+)
 from django.utils.functional import lazy
 from django.utils.html import format_html
 from django.utils.module_loading import import_string
-from django.utils.six import string_types, text_type
-from django.utils.translation import ugettext as _, ungettext
+from django.utils.translation import gettext as _, ngettext
 
 
-@lru_cache.lru_cache(maxsize=None)
+@functools.lru_cache(maxsize=None)
 def get_default_password_validators():
     return get_password_validators(settings.AUTH_PASSWORD_VALIDATORS)
 
@@ -86,10 +83,12 @@ def _password_validators_help_text_html(password_validators=None):
     help_texts = password_validators_help_texts(password_validators)
     help_items = [format_html('<li>{}</li>', help_text) for help_text in help_texts]
     return '<ul>%s</ul>' % ''.join(help_items) if help_items else ''
-password_validators_help_text_html = lazy(_password_validators_help_text_html, text_type)
 
 
-class MinimumLengthValidator(object):
+password_validators_help_text_html = lazy(_password_validators_help_text_html, str)
+
+
+class MinimumLengthValidator:
     """
     Validate whether the password is of a minimum length.
     """
@@ -99,7 +98,7 @@ class MinimumLengthValidator(object):
     def validate(self, password, user=None):
         if len(password) < self.min_length:
             raise ValidationError(
-                ungettext(
+                ngettext(
                     "This password is too short. It must contain at least %(min_length)d character.",
                     "This password is too short. It must contain at least %(min_length)d characters.",
                     self.min_length
@@ -109,14 +108,14 @@ class MinimumLengthValidator(object):
             )
 
     def get_help_text(self):
-        return ungettext(
+        return ngettext(
             "Your password must contain at least %(min_length)d character.",
             "Your password must contain at least %(min_length)d characters.",
             self.min_length
         ) % {'min_length': self.min_length}
 
 
-class UserAttributeSimilarityValidator(object):
+class UserAttributeSimilarityValidator:
     """
     Validate whether the password is sufficiently different from the user's
     attributes.
@@ -139,12 +138,15 @@ class UserAttributeSimilarityValidator(object):
 
         for attribute_name in self.user_attributes:
             value = getattr(user, attribute_name, None)
-            if not value or not isinstance(value, string_types):
+            if not value or not isinstance(value, str):
                 continue
-            value_parts = re.split('\W+', value) + [value]
+            value_parts = re.split(r'\W+', value) + [value]
             for value_part in value_parts:
-                if SequenceMatcher(a=password.lower(), b=value_part.lower()).quick_ratio() > self.max_similarity:
-                    verbose_name = force_text(user._meta.get_field(attribute_name).verbose_name)
+                if SequenceMatcher(a=password.lower(), b=value_part.lower()).quick_ratio() >= self.max_similarity:
+                    try:
+                        verbose_name = str(user._meta.get_field(attribute_name).verbose_name)
+                    except FieldDoesNotExist:
+                        verbose_name = attribute_name
                     raise ValidationError(
                         _("The password is too similar to the %(verbose_name)s."),
                         code='password_too_similar',
@@ -155,7 +157,7 @@ class UserAttributeSimilarityValidator(object):
         return _("Your password can't be too similar to your other personal information.")
 
 
-class CommonPasswordValidator(object):
+class CommonPasswordValidator:
     """
     Validate whether the password is a common password.
 
@@ -164,12 +166,13 @@ class CommonPasswordValidator(object):
     https://xato.net/passwords/more-top-worst-passwords/
     """
     DEFAULT_PASSWORD_LIST_PATH = os.path.join(
-        os.path.dirname(os.path.realpath(upath(__file__))), 'common-passwords.txt.gz'
+        os.path.dirname(os.path.realpath(__file__)), 'common-passwords.txt.gz'
     )
 
     def __init__(self, password_list_path=DEFAULT_PASSWORD_LIST_PATH):
         try:
-            common_passwords_lines = gzip.open(password_list_path).read().decode('utf-8').splitlines()
+            with gzip.open(password_list_path) as f:
+                common_passwords_lines = f.read().decode().splitlines()
         except IOError:
             with open(password_list_path) as f:
                 common_passwords_lines = f.readlines()
@@ -187,7 +190,7 @@ class CommonPasswordValidator(object):
         return _("Your password can't be a commonly used password.")
 
 
-class NumericPasswordValidator(object):
+class NumericPasswordValidator:
     """
     Validate whether the password is alphanumeric.
     """
