@@ -1,7 +1,6 @@
-class WKTAdapter(object):
+class WKTAdapter:
     """
-    This provides an adaptor for Geometries sent to the
-    MySQL and Oracle database backends.
+    An adaptor for Geometries sent to the MySQL and Oracle database backends.
     """
     def __init__(self, geom):
         self.wkt = geom.wkt
