@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import errno
 import os
 import re
@@ -9,10 +7,10 @@ from datetime import datetime
 
 from django.conf import settings
 from django.core.management.base import BaseCommand, CommandError
-from django.core.servers.basehttp import get_internal_wsgi_application, run
-from django.utils import autoreload, six
-from django.utils.encoding import force_text, get_system_encoding
-
+from django.core.servers.basehttp import (
+    WSGIServer, get_internal_wsgi_application, run,
+)
+from django.utils import autoreload
 
 naiveip_re = re.compile(r"""^(?:
 (?P<addr>
@@ -28,8 +26,13 @@ class Command(BaseCommand):
     # Validation is called explicitly each time the server is reloaded.
     requires_system_checks = False
     leave_locale_alone = True
+    stealth_options = ('shutdown_message',)
 
+    default_addr = '127.0.0.1'
+    default_addr_ipv6 = '::1'
     default_port = '8000'
+    protocol = 'http'
+    server_cls = WSGIServer
 
     def add_arguments(self, parser):
         parser.add_argument(
@@ -37,15 +40,15 @@ class Command(BaseCommand):
             help='Optional port number, or ipaddr:port'
         )
         parser.add_argument(
-            '--ipv6', '-6', action='store_true', dest='use_ipv6', default=False,
+            '--ipv6', '-6', action='store_true', dest='use_ipv6',
             help='Tells Django to use an IPv6 address.',
         )
         parser.add_argument(
-            '--nothreading', action='store_false', dest='use_threading', default=True,
+            '--nothreading', action='store_false', dest='use_threading',
             help='Tells Django to NOT use threading.',
         )
         parser.add_argument(
-            '--noreload', action='store_false', dest='use_reloader', default=True,
+            '--noreload', action='store_false', dest='use_reloader',
             help='Tells Django to NOT use the auto-reloader.',
         )
 
@@ -54,13 +57,11 @@ class Command(BaseCommand):
             # We rely on the environment because it's currently the only
             # way to reach WSGIRequestHandler. This seems an acceptable
             # compromise considering `runserver` runs indefinitely.
-            os.environ[str("DJANGO_COLORS")] = str("nocolor")
-        super(Command, self).execute(*args, **options)
+            os.environ["DJANGO_COLORS"] = "nocolor"
+        super().execute(*args, **options)
 
     def get_handler(self, *args, **options):
-        """
-        Returns the default WSGI handler for the runner.
-        """
+        """Return the default WSGI handler for the runner."""
         return get_internal_wsgi_application()
 
     def handle(self, *args, **options):
@@ -92,14 +93,12 @@ class Command(BaseCommand):
                 elif self.use_ipv6 and not _fqdn:
                     raise CommandError('"%s" is not a valid IPv6 address.' % self.addr)
         if not self.addr:
-            self.addr = '::1' if self.use_ipv6 else '127.0.0.1'
+            self.addr = self.default_addr_ipv6 if self.use_ipv6 else self.default_addr
             self._raw_ipv6 = self.use_ipv6
         self.run(**options)
 
     def run(self, **options):
-        """
-        Runs the server, using the autoreloader if needed
-        """
+        """Run the server, using the autoreloader if needed."""
         use_reloader = options['use_reloader']
 
         if use_reloader:
@@ -123,16 +122,15 @@ class Command(BaseCommand):
         # requires_migrations_check attribute.
         self.check_migrations()
         now = datetime.now().strftime('%B %d, %Y - %X')
-        if six.PY2:
-            now = now.decode(get_system_encoding())
         self.stdout.write(now)
         self.stdout.write((
             "Django version %(version)s, using settings %(settings)r\n"
-            "Starting development server at http://%(addr)s:%(port)s/\n"
+            "Starting development server at %(protocol)s://%(addr)s:%(port)s/\n"
             "Quit the server with %(quit_command)s.\n"
         ) % {
             "version": self.get_version(),
             "settings": settings.SETTINGS_MODULE,
+            "protocol": self.protocol,
             "addr": '[%s]' % self.addr if self._raw_ipv6 else self.addr,
             "port": self.port,
             "quit_command": quit_command,
@@ -141,7 +139,7 @@ class Command(BaseCommand):
         try:
             handler = self.get_handler(*args, **options)
             run(self.addr, int(self.port), handler,
-                ipv6=self.use_ipv6, threading=threading)
+                ipv6=self.use_ipv6, threading=threading, server_cls=self.server_cls)
         except socket.error as e:
             # Use helpful error messages instead of ugly tracebacks.
             ERRORS = {
@@ -152,7 +150,7 @@ class Command(BaseCommand):
             try:
                 error_text = ERRORS[e.errno]
             except KeyError:
-                error_text = force_text(e)
+                error_text = e
             self.stderr.write("Error: %s" % error_text)
             # Need to use an OS exit because sys.exit doesn't work in a thread
             os._exit(1)
@@ -161,5 +159,6 @@ class Command(BaseCommand):
                 self.stdout.write(shutdown_message)
             sys.exit(0)
 
+
 # Kept for backward compatibility
 BaseRunserverCommand = Command
