@@ -1,22 +1,20 @@
 import datetime
 import decimal
-import warnings
 from importlib import import_module
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
+from django.db import NotSupportedError, transaction
 from django.db.backends import utils
-from django.utils import six, timezone
+from django.utils import timezone
 from django.utils.dateparse import parse_duration
-from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.encoding import force_text
 
 
-class BaseDatabaseOperations(object):
+class BaseDatabaseOperations:
     """
-    This class encapsulates all backend-specific differences, such as the way
-    a backend performs ordering or calculates the ID of a recently-inserted
-    row.
+    Encapsulate backend-specific differences, such as the way a backend
+    performs ordering or calculates the ID of a recently-inserted row.
     """
     compiler_module = "django.db.models.sql.compiler"
 
@@ -34,6 +32,19 @@ class BaseDatabaseOperations(object):
         'intersection': 'INTERSECT',
         'difference': 'EXCEPT',
     }
+    # Mapping of Field.get_internal_type() (typically the model field's class
+    # name) to the data type to use for the Cast() function, if different from
+    # DatabaseWrapper.data_types.
+    cast_data_types = {}
+    # CharField data type if the max_length argument isn't provided.
+    cast_char_field_without_max_length = None
+
+    # Start and end points for window expressions.
+    PRECEDING = 'PRECEDING'
+    FOLLOWING = 'FOLLOWING'
+    UNBOUNDED_PRECEDING = 'UNBOUNDED ' + PRECEDING
+    UNBOUNDED_FOLLOWING = 'UNBOUNDED ' + FOLLOWING
+    CURRENT_ROW = 'CURRENT ROW'
 
     def __init__(self, connection):
         self.connection = connection
@@ -41,7 +52,7 @@ class BaseDatabaseOperations(object):
 
     def autoinc_sql(self, table, column):
         """
-        Returns any SQL needed to support auto-incrementing primary keys, or
+        Return any SQL needed to support auto-incrementing primary keys, or
         None if no SQL is necessary.
 
         This SQL is executed when a table is created.
@@ -50,7 +61,7 @@ class BaseDatabaseOperations(object):
 
     def bulk_batch_size(self, fields, objs):
         """
-        Returns the maximum allowed batch size for the backend. The fields
+        Return the maximum allowed batch size for the backend. The fields
         are the fields going to be inserted in the batch, the objs contains
         all the objects to be inserted.
         """
@@ -58,7 +69,7 @@ class BaseDatabaseOperations(object):
 
     def cache_key_culling_sql(self):
         """
-        Returns an SQL query that retrieves the first cache key greater than the
+        Return an SQL query that retrieves the first cache key greater than the
         n smallest.
 
         This is used by the 'db' cache backend to determine where to start
@@ -68,28 +79,28 @@ class BaseDatabaseOperations(object):
 
     def unification_cast_sql(self, output_field):
         """
-        Given a field instance, returns the SQL necessary to cast the result of
-        a union to that type. Note that the resulting string should contain a
-        '%s' placeholder for the expression being cast.
+        Given a field instance, return the SQL that casts the result of a union
+        to that type. The resulting string should contain a '%s' placeholder
+        for the expression being cast.
         """
         return '%s'
 
     def date_extract_sql(self, lookup_type, field_name):
         """
-        Given a lookup_type of 'year', 'month' or 'day', returns the SQL that
+        Given a lookup_type of 'year', 'month', or 'day', return the SQL that
         extracts a value from the given date field field_name.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a date_extract_sql() method')
 
     def date_interval_sql(self, timedelta):
         """
-        Implements the date interval functionality for expressions
+        Implement the date interval functionality for expressions.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a date_interval_sql() method')
 
     def date_trunc_sql(self, lookup_type, field_name):
         """
-        Given a lookup_type of 'year', 'month' or 'day', returns the SQL that
+        Given a lookup_type of 'year', 'month', or 'day', return the SQL that
         truncates the given date field field_name to a date object with only
         the given specificity.
         """
@@ -97,36 +108,35 @@ class BaseDatabaseOperations(object):
 
     def datetime_cast_date_sql(self, field_name, tzname):
         """
-        Returns the SQL necessary to cast a datetime value to date value.
+        Return the SQL to cast a datetime value to date value.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a datetime_cast_date() method')
 
     def datetime_cast_time_sql(self, field_name, tzname):
         """
-        Returns the SQL necessary to cast a datetime value to time value.
+        Return the SQL to cast a datetime value to time value.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a datetime_cast_time_sql() method')
 
     def datetime_extract_sql(self, lookup_type, field_name, tzname):
         """
-        Given a lookup_type of 'year', 'month', 'day', 'hour', 'minute' or
-        'second', returns the SQL that extracts a value from the given
-        datetime field field_name, and a tuple of parameters.
+        Given a lookup_type of 'year', 'month', 'day', 'hour', 'minute', or
+        'second', return the SQL that extracts a value from the given
+        datetime field field_name.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a datetime_extract_sql() method')
 
     def datetime_trunc_sql(self, lookup_type, field_name, tzname):
         """
-        Given a lookup_type of 'year', 'month', 'day', 'hour', 'minute' or
-        'second', returns the SQL that truncates the given datetime field
-        field_name to a datetime object with only the given specificity, and
-        a tuple of parameters.
+        Given a lookup_type of 'year', 'month', 'day', 'hour', 'minute', or
+        'second', return the SQL that truncates the given datetime field
+        field_name to a datetime object with only the given specificity.
         """
-        raise NotImplementedError('subclasses of BaseDatabaseOperations may require a datetime_trunk_sql() method')
+        raise NotImplementedError('subclasses of BaseDatabaseOperations may require a datetime_trunc_sql() method')
 
     def time_trunc_sql(self, lookup_type, field_name):
         """
-        Given a lookup_type of 'hour', 'minute' or 'second', returns the SQL
+        Given a lookup_type of 'hour', 'minute' or 'second', return the SQL
         that truncates the given time field field_name to a time object with
         only the given specificity.
         """
@@ -134,23 +144,23 @@ class BaseDatabaseOperations(object):
 
     def time_extract_sql(self, lookup_type, field_name):
         """
-        Given a lookup_type of 'hour', 'minute' or 'second', returns the SQL
+        Given a lookup_type of 'hour', 'minute', or 'second', return the SQL
         that extracts a value from the given time field field_name.
         """
         return self.date_extract_sql(lookup_type, field_name)
 
     def deferrable_sql(self):
         """
-        Returns the SQL necessary to make a constraint "initially deferred"
-        during a CREATE TABLE statement.
+        Return the SQL to make a constraint "initially deferred" during a
+        CREATE TABLE statement.
         """
         return ''
 
     def distinct_sql(self, fields):
         """
-        Returns an SQL DISTINCT clause which removes duplicate rows from the
-        result set. If any fields are given, only the given fields are being
-        checked for duplicates.
+        Return an SQL DISTINCT clause which removes duplicate rows from the
+        result set. If any fields are given, only check the given fields for
+        duplicates.
         """
         if fields:
             raise NotImplementedError('DISTINCT ON fields is not supported by this database backend')
@@ -160,119 +170,107 @@ class BaseDatabaseOperations(object):
     def fetch_returned_insert_id(self, cursor):
         """
         Given a cursor object that has just performed an INSERT...RETURNING
-        statement into a table that has an auto-incrementing ID, returns the
+        statement into a table that has an auto-incrementing ID, return the
         newly created ID.
         """
         return cursor.fetchone()[0]
 
     def field_cast_sql(self, db_type, internal_type):
         """
-        Given a column type (e.g. 'BLOB', 'VARCHAR'), and an internal type
-        (e.g. 'GenericIPAddressField'), returns the SQL necessary to cast it
-        before using it in a WHERE statement. Note that the resulting string
-        should contain a '%s' placeholder for the column being searched against.
+        Given a column type (e.g. 'BLOB', 'VARCHAR') and an internal type
+        (e.g. 'GenericIPAddressField'), return the SQL to cast it before using
+        it in a WHERE statement. The resulting string should contain a '%s'
+        placeholder for the column being searched against.
         """
         return '%s'
 
     def force_no_ordering(self):
         """
-        Returns a list used in the "ORDER BY" clause to force no ordering at
-        all. Returning an empty list means that nothing will be included in the
-        ordering.
+        Return a list used in the "ORDER BY" clause to force no ordering at
+        all. Return an empty list to include nothing in the ordering.
         """
         return []
 
-    def for_update_sql(self, nowait=False, skip_locked=False):
+    def for_update_sql(self, nowait=False, skip_locked=False, of=()):
         """
-        Returns the FOR UPDATE SQL clause to lock rows for an update operation.
+        Return the FOR UPDATE SQL clause to lock rows for an update operation.
         """
-        if nowait:
-            return 'FOR UPDATE NOWAIT'
-        elif skip_locked:
-            return 'FOR UPDATE SKIP LOCKED'
-        else:
-            return 'FOR UPDATE'
-
-    def fulltext_search_sql(self, field_name):
-        """
-        Returns the SQL WHERE clause to use in order to perform a full-text
-        search of the given field_name. Note that the resulting string should
-        contain a '%s' placeholder for the value being searched against.
-        """
-        # RemovedInDjango20Warning
-        raise NotImplementedError('Full-text search is not implemented for this database backend')
+        return 'FOR UPDATE%s%s%s' % (
+            ' OF %s' % ', '.join(of) if of else '',
+            ' NOWAIT' if nowait else '',
+            ' SKIP LOCKED' if skip_locked else '',
+        )
 
     def last_executed_query(self, cursor, sql, params):
         """
-        Returns a string of the query last executed by the given cursor, with
+        Return a string of the query last executed by the given cursor, with
         placeholders replaced with actual values.
 
-        `sql` is the raw query containing placeholders, and `params` is the
+        `sql` is the raw query containing placeholders and `params` is the
         sequence of parameters. These are used by default, but this method
         exists for database backends to provide a better implementation
         according to their own quoting schemes.
         """
-        # Convert params to contain Unicode values.
-        def to_unicode(s):
+        # Convert params to contain string values.
+        def to_string(s):
             return force_text(s, strings_only=True, errors='replace')
         if isinstance(params, (list, tuple)):
-            u_params = tuple(to_unicode(val) for val in params)
+            u_params = tuple(to_string(val) for val in params)
         elif params is None:
             u_params = ()
         else:
-            u_params = {to_unicode(k): to_unicode(v) for k, v in params.items()}
+            u_params = {to_string(k): to_string(v) for k, v in params.items()}
 
-        return six.text_type("QUERY = %r - PARAMS = %r") % (sql, u_params)
+        return "QUERY = %r - PARAMS = %r" % (sql, u_params)
 
     def last_insert_id(self, cursor, table_name, pk_name):
         """
         Given a cursor object that has just performed an INSERT statement into
-        a table that has an auto-incrementing ID, returns the newly created ID.
+        a table that has an auto-incrementing ID, return the newly created ID.
 
-        This method also receives the table name and the name of the primary-key
-        column.
+        `pk_name` is the name of the primary-key column.
         """
         return cursor.lastrowid
 
     def lookup_cast(self, lookup_type, internal_type=None):
         """
-        Returns the string to use in a query when performing lookups
-        ("contains", "like", etc.). The resulting string should contain a '%s'
-        placeholder for the column being searched against.
+        Return the string to use in a query when performing lookups
+        ("contains", "like", etc.). It should contain a '%s' placeholder for
+        the column being searched against.
         """
         return "%s"
 
     def max_in_list_size(self):
         """
-        Returns the maximum number of items that can be passed in a single 'IN'
+        Return the maximum number of items that can be passed in a single 'IN'
         list condition, or None if the backend does not impose a limit.
         """
         return None
 
     def max_name_length(self):
         """
-        Returns the maximum length of table and column names, or None if there
+        Return the maximum length of table and column names, or None if there
         is no limit.
         """
         return None
 
     def no_limit_value(self):
         """
-        Returns the value to use for the LIMIT when we are wanting "LIMIT
-        infinity". Returns None if the limit clause can be omitted in this case.
+        Return the value to use for the LIMIT when we are wanting "LIMIT
+        infinity". Return None if the limit clause can be omitted in this case.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a no_limit_value() method')
 
     def pk_default_value(self):
         """
-        Returns the value to use during an INSERT statement to specify that
+        Return the value to use during an INSERT statement to specify that
         the field should use its default value.
         """
         return 'DEFAULT'
 
     def prepare_sql_script(self, sql):
         """
-        Takes an SQL script that may contain multiple lines and returns a list
+        Take an SQL script that may contain multiple lines and return a list
         of statements to feed to successive cursor.execute() calls.
 
         Since few databases are able to process raw SQL scripts in a single
@@ -292,23 +290,23 @@ class BaseDatabaseOperations(object):
 
     def process_clob(self, value):
         """
-        Returns the value of a CLOB column, for backends that return a locator
+        Return the value of a CLOB column, for backends that return a locator
         object that requires additional processing.
         """
         return value
 
     def return_insert_id(self):
         """
-        For backends that support returning the last insert ID as part
-        of an insert query, this method returns the SQL and params to
-        append to the INSERT query. The returned fragment should
-        contain a format string to hold the appropriate column.
+        For backends that support returning the last insert ID as part of an
+        insert query, return the SQL and params to append to the INSERT query.
+        The returned fragment should contain a format string to hold the
+        appropriate column.
         """
         pass
 
     def compiler(self, compiler_name):
         """
-        Returns the SQLCompiler class corresponding to the given name,
+        Return the SQLCompiler class corresponding to the given name,
         in the namespace corresponding to the `compiler_module` attribute
         on this backend.
         """
@@ -318,31 +316,29 @@ class BaseDatabaseOperations(object):
 
     def quote_name(self, name):
         """
-        Returns a quoted version of the given table, index or column name. Does
+        Return a quoted version of the given table, index, or column name. Do
         not quote the given name if it's already been quoted.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a quote_name() method')
 
     def random_function_sql(self):
-        """
-        Returns an SQL expression that returns a random value.
-        """
+        """Return an SQL expression that returns a random value."""
         return 'RANDOM()'
 
     def regex_lookup(self, lookup_type):
         """
-        Returns the string to use in a query when performing regular expression
-        lookups (using "regex" or "iregex"). The resulting string should
-        contain a '%s' placeholder for the column being searched against.
+        Return the string to use in a query when performing regular expression
+        lookups (using "regex" or "iregex"). It should contain a '%s'
+        placeholder for the column being searched against.
 
-        If the feature is not supported (or part of it is not supported), a
-        NotImplementedError exception can be raised.
+        If the feature is not supported (or part of it is not supported), raise
+        NotImplementedError.
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations may require a regex_lookup() method')
 
     def savepoint_create_sql(self, sid):
         """
-        Returns the SQL for starting a new savepoint. Only required if the
+        Return the SQL for starting a new savepoint. Only required if the
         "uses_savepoints" feature is True. The "sid" parameter is a string
         for the savepoint id.
         """
@@ -350,32 +346,30 @@ class BaseDatabaseOperations(object):
 
     def savepoint_commit_sql(self, sid):
         """
-        Returns the SQL for committing the given savepoint.
+        Return the SQL for committing the given savepoint.
         """
         return "RELEASE SAVEPOINT %s" % self.quote_name(sid)
 
     def savepoint_rollback_sql(self, sid):
         """
-        Returns the SQL for rolling back the given savepoint.
+        Return the SQL for rolling back the given savepoint.
         """
         return "ROLLBACK TO SAVEPOINT %s" % self.quote_name(sid)
 
     def set_time_zone_sql(self):
         """
-        Returns the SQL that will set the connection's time zone.
+        Return the SQL that will set the connection's time zone.
 
-        Returns '' if the backend doesn't support time zones.
+        Return '' if the backend doesn't support time zones.
         """
         return ''
 
     def sql_flush(self, style, tables, sequences, allow_cascade=False):
         """
-        Returns a list of SQL statements required to remove all data from
+        Return a list of SQL statements required to remove all data from
         the given database tables (without actually removing the tables
-        themselves).
-
-        The returned value also includes SQL statements required to reset DB
-        sequences passed in :param sequences:.
+        themselves) and the SQL statements required to reset the sequences
+        passed in `sequences`.
 
         The `style` argument is a Style object as returned by either
         color_style() or no_style() in django.core.management.color.
@@ -386,10 +380,17 @@ class BaseDatabaseOperations(object):
         """
         raise NotImplementedError('subclasses of BaseDatabaseOperations must provide an sql_flush() method')
 
+    def execute_sql_flush(self, using, sql_list):
+        """Execute a list of SQL statements to flush the database."""
+        with transaction.atomic(using=using, savepoint=self.connection.features.can_rollback_ddl):
+            with self.connection.cursor() as cursor:
+                for sql in sql_list:
+                    cursor.execute(sql)
+
     def sequence_reset_by_name_sql(self, style, sequences):
         """
-        Returns a list of the SQL statements required to reset sequences
-        passed in :param sequences:.
+        Return a list of the SQL statements required to reset sequences
+        passed in `sequences`.
 
         The `style` argument is a Style object as returned by either
         color_style() or no_style() in django.core.management.color.
@@ -398,7 +399,7 @@ class BaseDatabaseOperations(object):
 
     def sequence_reset_sql(self, style, model_list):
         """
-        Returns a list of the SQL statements required to reset sequences for
+        Return a list of the SQL statements required to reset sequences for
         the given models.
 
         The `style` argument is a Style object as returned by either
@@ -407,33 +408,29 @@ class BaseDatabaseOperations(object):
         return []  # No sequence reset required by default.
 
     def start_transaction_sql(self):
-        """
-        Returns the SQL statement required to start a transaction.
-        """
+        """Return the SQL statement required to start a transaction."""
         return "BEGIN;"
 
     def end_transaction_sql(self, success=True):
-        """
-        Returns the SQL statement required to end a transaction.
-        """
+        """Return the SQL statement required to end a transaction."""
         if not success:
             return "ROLLBACK;"
         return "COMMIT;"
 
     def tablespace_sql(self, tablespace, inline=False):
         """
-        Returns the SQL that will be used in a query to define the tablespace.
+        Return the SQL that will be used in a query to define the tablespace.
 
-        Returns '' if the backend doesn't support tablespaces.
+        Return '' if the backend doesn't support tablespaces.
 
-        If inline is True, the SQL is appended to a row; otherwise it's appended
-        to the entire CREATE TABLE or CREATE INDEX statement.
+        If `inline` is True, append the SQL to a row; otherwise append it to
+        the entire CREATE TABLE or CREATE INDEX statement.
         """
         return ''
 
     def prep_for_like_query(self, x):
-        """Prepares a value for use in a LIKE query."""
-        return force_text(x).replace("\\", "\\\\").replace("%", r"\%").replace("_", r"\_")
+        """Prepare a value for use in a LIKE query."""
+        return str(x).replace("\\", "\\\\").replace("%", r"\%").replace("_", r"\_")
 
     # Same as prep_for_like_query(), but called for "iexact" matches, which
     # need not necessarily be implemented using "LIKE" in the backend.
@@ -442,14 +439,14 @@ class BaseDatabaseOperations(object):
     def validate_autopk_value(self, value):
         """
         Certain backends do not accept some values for "serial" fields
-        (for example zero in MySQL). This method will raise a ValueError
-        if the value is invalid, otherwise returns validated value.
+        (for example zero in MySQL). Raise a ValueError if the value is
+        invalid, otherwise return the validated value.
         """
         return value
 
     def adapt_unknown_value(self, value):
         """
-        Transforms a value to something compatible with the backend driver.
+        Transform a value to something compatible with the backend driver.
 
         This method only depends on the type of the value. It's designed for
         cases where the target type isn't known, such as .raw() SQL queries.
@@ -468,50 +465,50 @@ class BaseDatabaseOperations(object):
 
     def adapt_datefield_value(self, value):
         """
-        Transforms a date value to an object compatible with what is expected
+        Transform a date value to an object compatible with what is expected
         by the backend driver for date columns.
         """
         if value is None:
             return None
-        return six.text_type(value)
+        return str(value)
 
     def adapt_datetimefield_value(self, value):
         """
-        Transforms a datetime value to an object compatible with what is expected
+        Transform a datetime value to an object compatible with what is expected
         by the backend driver for datetime columns.
         """
         if value is None:
             return None
-        return six.text_type(value)
+        return str(value)
 
     def adapt_timefield_value(self, value):
         """
-        Transforms a time value to an object compatible with what is expected
+        Transform a time value to an object compatible with what is expected
         by the backend driver for time columns.
         """
         if value is None:
             return None
         if timezone.is_aware(value):
             raise ValueError("Django does not support timezone-aware times.")
-        return six.text_type(value)
+        return str(value)
 
     def adapt_decimalfield_value(self, value, max_digits=None, decimal_places=None):
         """
-        Transforms a decimal.Decimal value to an object compatible with what is
+        Transform a decimal.Decimal value to an object compatible with what is
         expected by the backend driver for decimal (numeric) columns.
         """
         return utils.format_number(value, max_digits, decimal_places)
 
     def adapt_ipaddressfield_value(self, value):
         """
-        Transforms a string representation of an IP address into the expected
+        Transform a string representation of an IP address into the expected
         type for the backend driver.
         """
         return value or None
 
     def year_lookup_bounds_for_date_field(self, value):
         """
-        Returns a two-elements list with the lower and upper bound to be used
+        Return a two-elements list with the lower and upper bound to be used
         with a BETWEEN operator to query a DateField value using a year
         lookup.
 
@@ -525,7 +522,7 @@ class BaseDatabaseOperations(object):
 
     def year_lookup_bounds_for_datetime_field(self, value):
         """
-        Returns a two-elements list with the lower and upper bound to be used
+        Return a two-elements list with the lower and upper bound to be used
         with a BETWEEN operator to query a DateTimeField value using a year
         lookup.
 
@@ -543,26 +540,19 @@ class BaseDatabaseOperations(object):
 
     def get_db_converters(self, expression):
         """
-        Get a list of functions needed to convert field data.
+        Return a list of functions needed to convert field data.
 
         Some field types on some backends do not provide data in the correct
         format, this is the hook for converter functions.
         """
         return []
 
-    def convert_durationfield_value(self, value, expression, connection, context):
+    def convert_durationfield_value(self, value, expression, connection):
         if value is not None:
             value = str(decimal.Decimal(value) / decimal.Decimal(1000000))
             value = parse_duration(value)
         return value
 
-    def check_aggregate_support(self, aggregate_func):
-        warnings.warn(
-            "check_aggregate_support has been deprecated. Use "
-            "check_expression_support instead.",
-            RemovedInDjango20Warning, stacklevel=2)
-        return self.check_expression_support(aggregate_func)
-
     def check_expression_support(self, expression):
         """
         Check that the backend supports the provided expression.
@@ -575,10 +565,11 @@ class BaseDatabaseOperations(object):
         pass
 
     def combine_expression(self, connector, sub_expressions):
-        """Combine a list of subexpressions into a single expression, using
+        """
+        Combine a list of subexpressions into a single expression, using
         the provided connecting operator. This is required because operators
         can vary between backends (e.g., Oracle with %% and &) and between
-        subexpression types (e.g., date expressions)
+        subexpression types (e.g., date expressions).
         """
         conn = ' %s ' % connector
         return conn.join(sub_expressions)
@@ -594,7 +585,8 @@ class BaseDatabaseOperations(object):
         return '%s'
 
     def modify_insert_params(self, placeholder, params):
-        """Allow modification of insert parameters. Needed for Oracle Spatial
+        """
+        Allow modification of insert parameters. Needed for Oracle Spatial
         backend due to #10888.
         """
         return params
@@ -602,7 +594,7 @@ class BaseDatabaseOperations(object):
     def integer_field_range(self, internal_type):
         """
         Given an integer field internal type (e.g. 'PositiveIntegerField'),
-        returns a tuple of the (min_value, max_value) form representing the
+        return a tuple of the (min_value, max_value) form representing the
         range of the column type bound to the field.
         """
         return self.integer_field_ranges[internal_type]
@@ -613,3 +605,34 @@ class BaseDatabaseOperations(object):
             rhs_sql, rhs_params = rhs
             return "(%s - %s)" % (lhs_sql, rhs_sql), lhs_params + rhs_params
         raise NotImplementedError("This backend does not support %s subtraction." % internal_type)
+
+    def window_frame_start(self, start):
+        if isinstance(start, int):
+            if start < 0:
+                return '%d %s' % (abs(start), self.PRECEDING)
+            elif start == 0:
+                return self.CURRENT_ROW
+        elif start is None:
+            return self.UNBOUNDED_PRECEDING
+        raise ValueError("start argument must be a negative integer, zero, or None, but got '%s'." % start)
+
+    def window_frame_end(self, end):
+        if isinstance(end, int):
+            if end == 0:
+                return self.CURRENT_ROW
+            elif end > 0:
+                return '%d %s' % (end, self.FOLLOWING)
+        elif end is None:
+            return self.UNBOUNDED_FOLLOWING
+        raise ValueError("end argument must be a positive integer, zero, or None, but got '%s'." % end)
+
+    def window_frame_rows_start_end(self, start=None, end=None):
+        """
+        Return SQL for start and end points in an OVER clause window frame.
+        """
+        if not self.connection.features.supports_over_clause:
+            raise NotSupportedError('This backend does not support window expressions.')
+        return self.window_frame_start(start), self.window_frame_end(end)
+
+    def window_frame_range_start_end(self, start=None, end=None):
+        return self.window_frame_rows_start_end(start, end)
