@@ -3,21 +3,15 @@ Oracle database backend for Django.
 
 Requires cx_Oracle: http://cx-oracle.sourceforge.net/
 """
-from __future__ import unicode_literals
-
 import datetime
 import decimal
 import os
 import platform
-import sys
-import warnings
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.db import utils
 from django.db.backends.base.base import BaseDatabaseWrapper
-from django.utils import six, timezone
-from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.encoding import force_bytes, force_text
 from django.utils.functional import cached_property
 
@@ -61,9 +55,10 @@ from .introspection import DatabaseIntrospection            # NOQA isort:skip
 from .operations import DatabaseOperations                  # NOQA isort:skip
 from .schema import DatabaseSchemaEditor                    # NOQA isort:skip
 from .utils import Oracle_datetime                          # NOQA isort:skip
+from .validation import DatabaseValidation                  # NOQA isort:skip
 
 
-class _UninitializedOperatorsDescriptor(object):
+class _UninitializedOperatorsDescriptor:
 
     def __get__(self, instance, cls=None):
         # If connection.operators is looked up before a connection has been
@@ -78,6 +73,7 @@ class _UninitializedOperatorsDescriptor(object):
 
 class DatabaseWrapper(BaseDatabaseWrapper):
     vendor = 'oracle'
+    display_name = 'Oracle'
     # This dictionary maps Field objects to their associated Oracle column
     # types, as strings. Column-type strings can contain format strings; they'll
     # be interpolated against the values of Field.__dict__ before being output.
@@ -86,12 +82,11 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     # Any format strings starting with "qn_" are quoted before being used in the
     # output (the "qn_" prefix is stripped before the lookup is performed.
     data_types = {
-        'AutoField': 'NUMBER(11)',
-        'BigAutoField': 'NUMBER(19)',
+        'AutoField': 'NUMBER(11) GENERATED BY DEFAULT ON NULL AS IDENTITY',
+        'BigAutoField': 'NUMBER(19) GENERATED BY DEFAULT ON NULL AS IDENTITY',
         'BinaryField': 'BLOB',
         'BooleanField': 'NUMBER(1)',
         'CharField': 'NVARCHAR2(%(max_length)s)',
-        'CommaSeparatedIntegerField': 'VARCHAR2(%(max_length)s)',
         'DateField': 'DATE',
         'DateTimeField': 'TIMESTAMP',
         'DecimalField': 'NUMBER(%(max_digits)s, %(decimal_places)s)',
@@ -116,11 +111,14 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     }
     data_type_check_constraints = {
         'BooleanField': '%(qn_column)s IN (0,1)',
-        'NullBooleanField': '(%(qn_column)s IN (0,1)) OR (%(qn_column)s IS NULL)',
+        'NullBooleanField': '%(qn_column)s IN (0,1)',
         'PositiveIntegerField': '%(qn_column)s >= 0',
         'PositiveSmallIntegerField': '%(qn_column)s >= 0',
     }
 
+    # Oracle doesn't support a database index on these columns.
+    _limited_data_types = ('clob', 'nclob', 'blob')
+
     operators = _UninitializedOperatorsDescriptor()
 
     _standard_operators = {
@@ -180,9 +178,10 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     features_class = DatabaseFeatures
     introspection_class = DatabaseIntrospection
     ops_class = DatabaseOperations
+    validation_class = DatabaseValidation
 
     def __init__(self, *args, **kwargs):
-        super(DatabaseWrapper, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         use_returning_into = self.settings_dict["OPTIONS"].get('use_returning_into', True)
         self.features.can_return_id_from_insert = use_returning_into
 
@@ -200,12 +199,9 @@ class DatabaseWrapper(BaseDatabaseWrapper):
                              settings_dict['PASSWORD'], dsn)
 
     def get_connection_params(self):
-        # Specify encoding to support unicode in DSN.
-        conn_params = {'encoding': 'UTF-8', 'nencoding': 'UTF-8'}
-        user_params = self.settings_dict['OPTIONS'].copy()
-        if 'use_returning_into' in user_params:
-            del user_params['use_returning_into']
-        conn_params.update(user_params)
+        conn_params = self.settings_dict['OPTIONS'].copy()
+        if 'use_returning_into' in conn_params:
+            del conn_params['use_returning_into']
         return conn_params
 
     def get_new_connection(self, conn_params):
@@ -269,7 +265,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
                 x = e.args[0]
                 if hasattr(x, 'code') and hasattr(x, 'message') \
                    and x.code == 2091 and 'ORA-02291' in x.message:
-                    six.reraise(utils.IntegrityError, utils.IntegrityError(*tuple(e.args)), sys.exc_info()[2])
+                    raise utils.IntegrityError(*tuple(e.args))
                 raise
 
     # Oracle doesn't support releasing savepoints. But we fake them when query
@@ -287,8 +283,8 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
     def check_constraints(self, table_names=None):
         """
-        To check constraints, we set constraints to immediate. Then, when, we're done we must ensure they
-        are returned to deferred.
+        Check constraints by setting them to immediate. Return them to deferred
+        afterward.
         """
         self.cursor().execute('SET CONSTRAINTS ALL IMMEDIATE')
         self.cursor().execute('SET CONSTRAINTS ALL DEFERRED')
@@ -314,7 +310,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             return None
 
 
-class OracleParam(object):
+class OracleParam:
     """
     Wrapper object for formatting parameters for Oracle. If the string
     representation of the value is large enough (greater than 4000 characters)
@@ -329,18 +325,10 @@ class OracleParam(object):
         # without being converted by DateTimeField.get_db_prep_value.
         if settings.USE_TZ and (isinstance(param, datetime.datetime) and
                                 not isinstance(param, Oracle_datetime)):
-            if timezone.is_aware(param):
-                warnings.warn(
-                    "The Oracle database adapter received an aware datetime (%s), "
-                    "probably from cursor.execute(). Update your code to pass a "
-                    "naive datetime in the database connection's time zone (UTC by "
-                    "default).", RemovedInDjango20Warning)
-                param = param.astimezone(timezone.utc).replace(tzinfo=None)
             param = Oracle_datetime.from_datetime(param)
 
         string_size = 0
-        # Oracle doesn't recognize True and False correctly in Python 3.
-        # The conversion done below works both in 2 and 3.
+        # Oracle doesn't recognize True and False correctly.
         if param is True:
             param = 1
         elif param is False:
@@ -353,7 +341,7 @@ class OracleParam(object):
             # To transmit to the database, we need Unicode if supported
             # To get size right, we must consider bytes.
             self.force_bytes = force_text(param, cursor.charset, strings_only)
-            if isinstance(self.force_bytes, six.string_types):
+            if isinstance(self.force_bytes, str):
                 # We could optimize by only converting up to 4000 bytes here
                 string_size = len(force_bytes(param, cursor.charset, strings_only))
         if hasattr(param, 'input_size'):
@@ -362,13 +350,11 @@ class OracleParam(object):
         elif string_size > 4000:
             # Mark any string param greater than 4000 characters as a CLOB.
             self.input_size = Database.CLOB
-        elif isinstance(param, decimal.Decimal):
-            self.input_size = Database.NUMBER
         else:
             self.input_size = None
 
 
-class VariableWrapper(object):
+class VariableWrapper:
     """
     An adapter class for cursor variables that prevents the wrapped object
     from being converted into a string when used to instantiate an OracleParam.
@@ -392,7 +378,7 @@ class VariableWrapper(object):
             setattr(self.var, key, value)
 
 
-class FormatStylePlaceholderCursor(object):
+class FormatStylePlaceholderCursor:
     """
     Django uses "format" (e.g. '%s') style placeholders, but Oracle uses ":var"
     style. This fixes it -- but note that if you want to use a literal "%s" in
@@ -406,21 +392,44 @@ class FormatStylePlaceholderCursor(object):
     def __init__(self, connection):
         self.cursor = connection.cursor()
         self.cursor.outputtypehandler = self._output_type_handler
-        # Default arraysize of 1 is highly sub-optimal.
+        # The default for cx_Oracle < 5.3 is 50.
         self.cursor.arraysize = 100
 
+    @staticmethod
+    def _output_number_converter(value):
+        return decimal.Decimal(value) if '.' in value else int(value)
+
     @staticmethod
     def _output_type_handler(cursor, name, defaultType, length, precision, scale):
         """
-        Called for each db column fetched from cursors. Return numbers as
-        strings so that decimal values don't have rounding error.
+        Called for each db column fetched from cursors. Return numbers as the
+        appropriate Python type.
         """
         if defaultType == Database.NUMBER:
+            if scale == -127:
+                if precision == 0:
+                    # NUMBER column: decimal-precision floating point.
+                    # This will normally be an integer from a sequence,
+                    # but it could be a decimal value.
+                    outconverter = FormatStylePlaceholderCursor._output_number_converter
+                else:
+                    # FLOAT column: binary-precision floating point.
+                    # This comes from FloatField columns.
+                    outconverter = float
+            elif precision > 0:
+                # NUMBER(p,s) column: decimal-precision fixed point.
+                # This comes from IntegerField and DecimalField columns.
+                outconverter = int if scale == 0 else decimal.Decimal
+            else:
+                # No type information. This normally comes from a
+                # mathematical expression in the SELECT list. Guess int
+                # or Decimal based on whether it has a decimal point.
+                outconverter = FormatStylePlaceholderCursor._output_number_converter
             return cursor.var(
                 Database.STRING,
                 size=255,
                 arraysize=cursor.arraysize,
-                outconverter=str,
+                outconverter=outconverter,
             )
 
     def _format_params(self, params):
@@ -463,33 +472,27 @@ class FormatStylePlaceholderCursor(object):
             query = query[:-1]
         if params is None:
             params = []
-            query = query
         elif hasattr(params, 'keys'):
             # Handle params as dict
-            args = {k: ":%s" % k for k in params.keys()}
+            args = {k: ":%s" % k for k in params}
             query = query % args
         elif unify_by_values and len(params) > 0:
             # Handle params as a dict with unified query parameters by their
             # values. It can be used only in single query execute() because
             # executemany() shares the formatted query with each of the params
             # list. e.g. for input params = [0.75, 2, 0.75, 'sth', 0.75]
-            # params_dict = {
-            #   (2, <type 'int'>): ':arg2',
-            #   (0.75, <type 'float'>): ':arg1',
-            #   ('sth', <type 'str'>): ':arg0',
-            # }
+            # params_dict = {0.75: ':arg0', 2: ':arg1', 'sth': ':arg2'}
             # args = [':arg0', ':arg1', ':arg0', ':arg2', ':arg0']
             # params = {':arg0': 0.75, ':arg1': 2, ':arg2': 'sth'}
-            params = [(param, type(param)) for param in params]
             params_dict = {param: ':arg%d' % i for i, param in enumerate(set(params))}
             args = [params_dict[param] for param in params]
-            params = {value: key[0] for key, value in params_dict.items()}
+            params = {value: key for key, value in params_dict.items()}
             query = query % tuple(args)
         else:
             # Handle params as sequence
             args = [(':arg%d' % i) for i in range(len(params))]
             query = query % tuple(args)
-        return force_text(query, self.charset), self._format_params(params)
+        return query, self._format_params(params)
 
     def execute(self, query, params=None):
         query, params = self._fix_for_params(query, params, unify_by_values=True)
@@ -509,19 +512,13 @@ class FormatStylePlaceholderCursor(object):
         self._guess_input_sizes(formatted)
         return self.cursor.executemany(query, [self._param_generator(p) for p in formatted])
 
-    def fetchone(self):
-        row = self.cursor.fetchone()
-        if row is None:
-            return row
-        return _rowfactory(row, self.cursor)
-
     def fetchmany(self, size=None):
         if size is None:
             size = self.arraysize
-        return tuple(_rowfactory(r, self.cursor) for r in self.cursor.fetchmany(size))
+        return tuple(self.cursor.fetchmany(size))
 
     def fetchall(self):
-        return tuple(_rowfactory(r, self.cursor) for r in self.cursor.fetchall())
+        return tuple(self.cursor.fetchall())
 
     def close(self):
         try:
@@ -537,78 +534,7 @@ class FormatStylePlaceholderCursor(object):
         return VariableWrapper(self.cursor.arrayvar(*args))
 
     def __getattr__(self, attr):
-        if attr in self.__dict__:
-            return self.__dict__[attr]
-        else:
-            return getattr(self.cursor, attr)
+        return getattr(self.cursor, attr)
 
     def __iter__(self):
-        return CursorIterator(self.cursor)
-
-
-class CursorIterator(six.Iterator):
-    """
-    Cursor iterator wrapper that invokes our custom row factory.
-    """
-
-    def __init__(self, cursor):
-        self.cursor = cursor
-        self.iter = iter(cursor)
-
-    def __iter__(self):
-        return self
-
-    def __next__(self):
-        return _rowfactory(next(self.iter), self.cursor)
-
-
-def _rowfactory(row, cursor):
-    # Cast numeric values as the appropriate Python type based upon the
-    # cursor description, and convert strings to unicode.
-    casted = []
-    for value, desc in zip(row, cursor.description):
-        if value is not None and desc[1] is Database.NUMBER:
-            precision = desc[4] or 0
-            scale = desc[5] or 0
-            if scale == -127:
-                if precision == 0:
-                    # NUMBER column: decimal-precision floating point
-                    # This will normally be an integer from a sequence,
-                    # but it could be a decimal value.
-                    if '.' in value:
-                        value = decimal.Decimal(value)
-                    else:
-                        value = int(value)
-                else:
-                    # FLOAT column: binary-precision floating point.
-                    # This comes from FloatField columns.
-                    value = float(value)
-            elif precision > 0:
-                # NUMBER(p,s) column: decimal-precision fixed point.
-                # This comes from IntField and DecimalField columns.
-                if scale == 0:
-                    value = int(value)
-                else:
-                    value = decimal.Decimal(value)
-            elif '.' in value:
-                # No type information. This normally comes from a
-                # mathematical expression in the SELECT list. Guess int
-                # or Decimal based on whether it has a decimal point.
-                value = decimal.Decimal(value)
-            else:
-                value = int(value)
-        elif desc[1] in (Database.STRING, Database.FIXED_CHAR,
-                         Database.LONG_STRING):
-            value = to_unicode(value)
-        casted.append(value)
-    return tuple(casted)
-
-
-def to_unicode(s):
-    """
-    Convert strings to Unicode objects (and return all other data types
-    unchanged).
-    """
-    if isinstance(s, six.string_types):
-        return force_text(s)
-    return s
+        return iter(self.cursor)
