@@ -5,7 +5,6 @@ from django.db.backends.base.creation import BaseDatabaseCreation
 from django.db.utils import DatabaseError
 from django.utils.crypto import get_random_string
 from django.utils.functional import cached_property
-from django.utils.six.moves import input
 
 TEST_DATABASE_PREFIX = 'test_'
 
@@ -36,11 +35,10 @@ class DatabaseCreation(BaseDatabaseCreation):
             try:
                 self._execute_test_db_creation(cursor, parameters, verbosity, keepdb)
             except Exception as e:
-                # if we want to keep the db, then no need to do any of the below,
-                # just return and skip it all.
-                if keepdb:
-                    return
-                sys.stderr.write("Got an error creating the test database: %s\n" % e)
+                if 'ORA-01543' not in str(e):
+                    # All errors except "tablespace already exists" cancel tests
+                    sys.stderr.write("Got an error creating the test database: %s\n" % e)
+                    sys.exit(2)
                 if not autoclobber:
                     confirm = input(
                         "It appears the test database, %s, already exists. "
@@ -76,7 +74,10 @@ class DatabaseCreation(BaseDatabaseCreation):
             try:
                 self._create_test_user(cursor, parameters, verbosity, keepdb)
             except Exception as e:
-                sys.stderr.write("Got an error creating the test user: %s\n" % e)
+                if 'ORA-01920' not in str(e):
+                    # All errors except "user already exists" cancel tests
+                    sys.stderr.write("Got an error creating the test user: %s\n" % e)
+                    sys.exit(2)
                 if not autoclobber:
                     confirm = input(
                         "It appears the test user, %s, already exists. Type "
@@ -96,17 +97,20 @@ class DatabaseCreation(BaseDatabaseCreation):
                     print("Tests cancelled.")
                     sys.exit(1)
 
+        # Cursor must be closed before closing connection.
+        cursor.close()
         self._maindb_connection.close()  # done with main user -- test user and tablespaces created
         self._switch_to_test_user(parameters)
         return self.connection.settings_dict['NAME']
 
     def _switch_to_test_user(self, parameters):
         """
-        Oracle doesn't have the concept of separate databases under the same user.
-        Thus, we use a separate user (see _create_test_db). This method is used
-        to switch to that user. We will need the main user again for clean-up when
-        we end testing, so we keep its credentials in SAVED_USER/SAVED_PASSWORD
-        entries in the settings dict.
+        Switch to the user that's used for creating the test database.
+
+        Oracle doesn't have the concept of separate databases under the same
+        user, so a separate user is used; see _create_test_db(). The main user
+        is also needed for cleanup when testing is completed, so save its
+        credentials in the SAVED_USER/SAVED_PASSWORD key in the settings dict.
         """
         real_settings = settings.DATABASES[self.connection.alias]
         real_settings['SAVED_USER'] = self.connection.settings_dict['SAVED_USER'] = \
@@ -121,8 +125,8 @@ class DatabaseCreation(BaseDatabaseCreation):
 
     def set_as_test_mirror(self, primary_settings_dict):
         """
-        Set this database up to be used in testing as a mirror of a primary database
-        whose settings are given
+        Set this database up to be used in testing as a mirror of a primary
+        database whose settings are given.
         """
         self.connection.settings_dict['USER'] = primary_settings_dict['USER']
         self.connection.settings_dict['PASSWORD'] = primary_settings_dict['PASSWORD']
@@ -165,7 +169,7 @@ class DatabaseCreation(BaseDatabaseCreation):
     def _destroy_test_db(self, test_database_name, verbosity=1):
         """
         Destroy a test database, prompting the user for confirmation if the
-        database already exists. Returns the name of the test database created.
+        database already exists. Return the name of the test database created.
         """
         self.connection.settings_dict['USER'] = self.connection.settings_dict['SAVED_USER']
         self.connection.settings_dict['PASSWORD'] = self.connection.settings_dict['SAVED_PASSWORD']
@@ -180,6 +184,8 @@ class DatabaseCreation(BaseDatabaseCreation):
             if verbosity >= 1:
                 print('Destroying test database tables...')
             self._execute_test_db_destruction(cursor, parameters, verbosity)
+        # Cursor must be closed before closing connection.
+        cursor.close()
         self._maindb_connection.close()
 
     def _execute_test_db_creation(self, cursor, parameters, verbosity, keepdb=False):
@@ -187,12 +193,12 @@ class DatabaseCreation(BaseDatabaseCreation):
             print("_create_test_db(): dbname = %s" % parameters['user'])
         statements = [
             """CREATE TABLESPACE %(tblspace)s
-               DATAFILE '%(datafile)s' SIZE 20M
-               REUSE AUTOEXTEND ON NEXT 10M MAXSIZE %(maxsize)s
+               DATAFILE '%(datafile)s' SIZE %(size)s
+               REUSE AUTOEXTEND ON NEXT %(extsize)s MAXSIZE %(maxsize)s
             """,
             """CREATE TEMPORARY TABLESPACE %(tblspace_temp)s
-               TEMPFILE '%(datafile_tmp)s' SIZE 20M
-               REUSE AUTOEXTEND ON NEXT 10M MAXSIZE %(maxsize_tmp)s
+               TEMPFILE '%(datafile_tmp)s' SIZE %(size_tmp)s
+               REUSE AUTOEXTEND ON NEXT %(extsize_tmp)s MAXSIZE %(maxsize_tmp)s
             """,
         ]
         # Ignore "tablespace already exists" error when keepdb is on.
@@ -285,15 +291,18 @@ class DatabaseCreation(BaseDatabaseCreation):
             'tblspace_temp': self._test_database_tblspace_tmp(),
             'datafile': self._test_database_tblspace_datafile(),
             'datafile_tmp': self._test_database_tblspace_tmp_datafile(),
-            'maxsize': self._test_database_tblspace_size(),
-            'maxsize_tmp': self._test_database_tblspace_tmp_size(),
+            'maxsize': self._test_database_tblspace_maxsize(),
+            'maxsize_tmp': self._test_database_tblspace_tmp_maxsize(),
+            'size': self._test_database_tblspace_size(),
+            'size_tmp': self._test_database_tblspace_tmp_size(),
+            'extsize': self._test_database_tblspace_extsize(),
+            'extsize_tmp': self._test_database_tblspace_tmp_extsize(),
         }
 
     def _test_settings_get(self, key, default=None, prefixed=None):
         """
-        Return a value from the test settings dict,
-        or a given default,
-        or a prefixed entry from the main settings dict
+        Return a value from the test settings dict, or a given default, or a
+        prefixed entry from the main settings dict.
         """
         settings_dict = self.connection.settings_dict
         val = settings_dict['TEST'].get(key, default)
@@ -336,17 +345,29 @@ class DatabaseCreation(BaseDatabaseCreation):
         tblspace = '%s.dbf' % self._test_database_tblspace_tmp()
         return self._test_settings_get('DATAFILE_TMP', default=tblspace)
 
-    def _test_database_tblspace_size(self):
+    def _test_database_tblspace_maxsize(self):
         return self._test_settings_get('DATAFILE_MAXSIZE', default='500M')
 
-    def _test_database_tblspace_tmp_size(self):
+    def _test_database_tblspace_tmp_maxsize(self):
         return self._test_settings_get('DATAFILE_TMP_MAXSIZE', default='500M')
 
+    def _test_database_tblspace_size(self):
+        return self._test_settings_get('DATAFILE_SIZE', default='50M')
+
+    def _test_database_tblspace_tmp_size(self):
+        return self._test_settings_get('DATAFILE_TMP_SIZE', default='50M')
+
+    def _test_database_tblspace_extsize(self):
+        return self._test_settings_get('DATAFILE_EXTSIZE', default='25M')
+
+    def _test_database_tblspace_tmp_extsize(self):
+        return self._test_settings_get('DATAFILE_TMP_EXTSIZE', default='25M')
+
     def _get_test_db_name(self):
         """
-        We need to return the 'production' DB name to get the test DB creation
-        machinery to work. This isn't a great deal in this case because DB
-        names as handled by Django haven't real counterparts in Oracle.
+        Return the 'production' DB name to get the test DB creation machinery
+        to work. This isn't a great deal in this case because DB names as
+        handled by Django don't have real counterparts in Oracle.
         """
         return self.connection.settings_dict['NAME']
 
