@@ -1,12 +1,8 @@
-from __future__ import unicode_literals
-
-import sys
-
 from django.conf import settings
 from django.template import Library, Node, TemplateSyntaxError, Variable
 from django.template.base import TOKEN_TEXT, TOKEN_VAR, render_value_in_context
 from django.template.defaulttags import token_kwargs
-from django.utils import six, translation
+from django.utils import translation
 from django.utils.safestring import SafeData, mark_safe
 
 register = Library()
@@ -17,7 +13,7 @@ class GetAvailableLanguagesNode(Node):
         self.variable = variable
 
     def render(self, context):
-        context[self.variable] = [(k, translation.ugettext(v)) for k, v in settings.LANGUAGES]
+        context[self.variable] = [(k, translation.gettext(v)) for k, v in settings.LANGUAGES]
         return ''
 
 
@@ -76,7 +72,7 @@ class TranslateNode(Node):
         self.asvar = asvar
         self.message_context = message_context
         self.filter_expression = filter_expression
-        if isinstance(self.filter_expression.var, six.string_types):
+        if isinstance(self.filter_expression.var, str):
             self.filter_expression.var = Variable("'%s'" %
                                                   self.filter_expression.var)
 
@@ -146,13 +142,13 @@ class BlockTranslateNode(Node):
                 result = translation.npgettext(message_context, singular,
                                                plural, count)
             else:
-                result = translation.ungettext(singular, plural, count)
+                result = translation.ngettext(singular, plural, count)
             vars.extend(plural_vars)
         else:
             if message_context:
                 result = translation.pgettext(message_context, singular)
             else:
-                result = translation.ugettext(singular)
+                result = translation.gettext(singular)
         default_value = context.template.engine.string_if_invalid
 
         def render_value(key):
@@ -271,7 +267,7 @@ def language_name(lang_code):
 @register.filter
 def language_name_translated(lang_code):
     english_name = translation.get_language_info(lang_code)['name']
-    return translation.ugettext(english_name)
+    return translation.gettext(english_name)
 
 
 @register.filter
@@ -390,8 +386,9 @@ def do_translate(parser, token):
             try:
                 value = remaining.pop(0)
             except IndexError:
-                msg = "No argument provided to the '%s' tag for the context option." % bits[0]
-                six.reraise(TemplateSyntaxError, TemplateSyntaxError(msg), sys.exc_info()[2])
+                raise TemplateSyntaxError(
+                    "No argument provided to the '%s' tag for the context option." % bits[0]
+                )
             if value in invalid_context:
                 raise TemplateSyntaxError(
                     "Invalid argument '%s' provided to the '%s' tag for the context option" % (value, bits[0]),
@@ -401,8 +398,9 @@ def do_translate(parser, token):
             try:
                 value = remaining.pop(0)
             except IndexError:
-                msg = "No argument provided to the '%s' tag for the as option." % bits[0]
-                six.reraise(TemplateSyntaxError, TemplateSyntaxError(msg), sys.exc_info()[2])
+                raise TemplateSyntaxError(
+                    "No argument provided to the '%s' tag for the as option." % bits[0]
+                )
             asvar = value
         else:
             raise TemplateSyntaxError(
@@ -483,18 +481,18 @@ def do_block_translate(parser, token):
                 value = remaining_bits.pop(0)
                 value = parser.compile_filter(value)
             except Exception:
-                msg = (
-                    '"context" in %r tag expected '
-                    'exactly one argument.') % bits[0]
-                six.reraise(TemplateSyntaxError, TemplateSyntaxError(msg), sys.exc_info()[2])
+                raise TemplateSyntaxError(
+                    '"context" in %r tag expected exactly one argument.' % bits[0]
+                )
         elif option == "trimmed":
             value = True
         elif option == "asvar":
             try:
                 value = remaining_bits.pop(0)
             except IndexError:
-                msg = "No argument provided to the '%s' tag for the asvar option." % bits[0]
-                six.reraise(TemplateSyntaxError, TemplateSyntaxError(msg), sys.exc_info()[2])
+                raise TemplateSyntaxError(
+                    "No argument provided to the '%s' tag for the asvar option." % bits[0]
+                )
             asvar = value
         else:
             raise TemplateSyntaxError('Unknown argument for %r tag: %r.' %
@@ -502,7 +500,7 @@ def do_block_translate(parser, token):
         options[option] = value
 
     if 'count' in options:
-        countervar, counter = list(options['count'].items())[0]
+        countervar, counter = next(iter(options['count'].items()))
     else:
         countervar, counter = None, None
     if 'context' in options:
