@@ -1,25 +1,20 @@
 """HTML utilities suitable for global use."""
 
-from __future__ import unicode_literals
-
 import re
-import warnings
-
-from django.utils import six
-from django.utils.deprecation import RemovedInDjango110Warning
-from django.utils.encoding import force_str, force_text
-from django.utils.functional import allow_lazy
-from django.utils.http import RFC3986_GENDELIMS, RFC3986_SUBDELIMS
-from django.utils.safestring import SafeData, SafeText, mark_safe
-from django.utils.six.moves.urllib.parse import (
+from urllib.parse import (
     parse_qsl, quote, unquote, urlencode, urlsplit, urlunsplit,
 )
+
+from django.utils.encoding import force_text
+from django.utils.functional import Promise, keep_lazy, keep_lazy_text
+from django.utils.http import RFC3986_GENDELIMS, RFC3986_SUBDELIMS
+from django.utils.safestring import SafeData, SafeText, mark_safe
 from django.utils.text import normalize_newlines
 
 from .html_parser import HTMLParseError, HTMLParser
 
 # Configuration for urlize() function.
-TRAILING_PUNCTUATION = ['.', ',', ':', ';', '.)', '"', '\'', '!']
+TRAILING_PUNCTUATION_CHARS = '.,:;!'
 WRAPPING_PUNCTUATION = [('(', ')'), ('<', '>'), ('[', ']'), ('&lt;', '&gt;'), ('"', '"'), ('\'', '\'')]
 
 # List of possible strings used for bullets in bulleted lists.
@@ -29,29 +24,23 @@ unencoded_ampersands_re = re.compile(r'&(?!(\w+|#\d+);)')
 word_split_re = re.compile(r'''([\s<>"']+)''')
 simple_url_re = re.compile(r'^https?://\[?\w', re.IGNORECASE)
 simple_url_2_re = re.compile(r'^www\.|^(?!http)\w[^@]+\.(com|edu|gov|int|mil|net|org)($|/.*)$', re.IGNORECASE)
-simple_email_re = re.compile(r'^\S+@\S+\.\S+$')
-link_target_attribute_re = re.compile(r'(<a [^>]*?)target=[^\s>]+')
-html_gunk_re = re.compile(
-    r'(?:<br clear="all">|<i><\/i>|<b><\/b>|<em><\/em>|<strong><\/strong>|'
-    '<\/?smallcaps>|<\/?uppercase>)', re.IGNORECASE)
-hard_coded_bullets_re = re.compile(
-    r'((?:<p>(?:%s).*?[a-zA-Z].*?</p>\s*)+)' % '|'.join(re.escape(x)
-    for x in DOTS), re.DOTALL)
-trailing_empty_content_re = re.compile(r'(?:<p>(?:&nbsp;|\s|<br \/>)*?</p>\s*)+\Z')
 
 
+@keep_lazy(str, SafeText)
 def escape(text):
     """
-    Returns the given text with ampersands, quotes and angle brackets encoded
+    Return the given text with ampersands, quotes and angle brackets encoded
     for use in HTML.
 
-    This function always escapes its input, even if it's already escaped and
-    marked as such. This may result in double-escaping. If this is a concern,
-    use conditional_escape() instead.
+    Always escape input, even if it's already escaped and marked as such.
+    This may result in double-escaping. If this is a concern, use
+    conditional_escape() instead.
     """
-    return mark_safe(force_text(text).replace('&', '&amp;').replace('<', '&lt;')
-        .replace('>', '&gt;').replace('"', '&quot;').replace("'", '&#39;'))
-escape = allow_lazy(escape, six.text_type, SafeText)
+    return mark_safe(
+        str(text).replace('&', '&amp;').replace('<', '&lt;')
+        .replace('>', '&gt;').replace('"', '&quot;').replace("'", '&#39;')
+    )
+
 
 _js_escapes = {
     ord('\\'): '\\u005C',
@@ -63,6 +52,7 @@ _js_escapes = {
     ord('='): '\\u003D',
     ord('-'): '\\u002D',
     ord(';'): '\\u003B',
+    ord('`'): '\\u0060',
     ord('\u2028'): '\\u2028',
     ord('\u2029'): '\\u2029'
 }
@@ -71,10 +61,10 @@ _js_escapes = {
 _js_escapes.update((ord('%c' % z), '\\u%04X' % z) for z in range(32))
 
 
+@keep_lazy(str, SafeText)
 def escapejs(value):
-    """Hex encodes characters for use in JavaScript strings."""
-    return mark_safe(force_text(value).translate(_js_escapes))
-escapejs = allow_lazy(escapejs, six.text_type, SafeText)
+    """Hex encode characters for use in JavaScript strings."""
+    return mark_safe(str(value).translate(_js_escapes))
 
 
 def conditional_escape(text):
@@ -84,6 +74,8 @@ def conditional_escape(text):
     This function relies on the __html__ convention used both by Django's
     SafeData class and by third-party libraries like markupsafe.
     """
+    if isinstance(text, Promise):
+        text = str(text)
     if hasattr(text, '__html__'):
         return text.__html__()
     else:
@@ -92,12 +84,12 @@ def conditional_escape(text):
 
 def format_html(format_string, *args, **kwargs):
     """
-    Similar to str.format, but passes all arguments through conditional_escape,
-    and calls 'mark_safe' on the result. This function should be used instead
+    Similar to str.format, but pass all arguments through conditional_escape(),
+    and call mark_safe() on the result. This function should be used instead
     of str.format or % interpolation to build up small HTML fragments.
     """
     args_safe = map(conditional_escape, args)
-    kwargs_safe = {k: conditional_escape(v) for (k, v) in six.iteritems(kwargs)}
+    kwargs_safe = {k: conditional_escape(v) for (k, v) in kwargs.items()}
     return mark_safe(format_string.format(*args_safe, **kwargs_safe))
 
 
@@ -120,16 +112,16 @@ def format_html_join(sep, format_string, args_generator):
         for args in args_generator))
 
 
+@keep_lazy_text
 def linebreaks(value, autoescape=False):
-    """Converts newlines into <p> and <br />s."""
+    """Convert newlines into <p> and <br />s."""
     value = normalize_newlines(value)
-    paras = re.split('\n{2,}', value)
+    paras = re.split('\n{2,}', str(value))
     if autoescape:
         paras = ['<p>%s</p>' % escape(p).replace('\n', '<br />') for p in paras]
     else:
         paras = ['<p>%s</p>' % p.replace('\n', '<br />') for p in paras]
     return '\n\n'.join(paras)
-linebreaks = allow_lazy(linebreaks, six.text_type)
 
 
 class MLStripper(HTMLParser):
@@ -168,63 +160,35 @@ def _strip_once(value):
         return s.get_data()
 
 
+@keep_lazy_text
 def strip_tags(value):
-    """Returns the given HTML with all tags stripped."""
+    """Return the given HTML with all tags stripped."""
     # Note: in typical case this loop executes _strip_once once. Loop condition
     # is redundant, but helps to reduce number of executions of _strip_once.
+    value = force_text(value)
     while '<' in value and '>' in value:
         new_value = _strip_once(value)
         if len(new_value) >= len(value):
-            # _strip_once was not able to detect more tags or length increased
-            # due to http://bugs.python.org/issue20288
-            # (affects Python 2 < 2.7.7 and Python 3 < 3.3.5)
+            # _strip_once was not able to detect more tags
             break
         value = new_value
     return value
-strip_tags = allow_lazy(strip_tags)
-
-
-def remove_tags(html, tags):
-    """Returns the given HTML with given tags removed."""
-    warnings.warn(
-        "django.utils.html.remove_tags() and the removetags template filter "
-        "are deprecated. Consider using the bleach library instead.",
-        RemovedInDjango110Warning, stacklevel=3
-    )
-    tags = [re.escape(tag) for tag in tags.split()]
-    tags_re = '(%s)' % '|'.join(tags)
-    starttag_re = re.compile(r'<%s(/?>|(\s+[^>]*>))' % tags_re, re.U)
-    endtag_re = re.compile('</%s>' % tags_re)
-    html = starttag_re.sub('', html)
-    html = endtag_re.sub('', html)
-    return html
-remove_tags = allow_lazy(remove_tags, six.text_type)
 
 
+@keep_lazy_text
 def strip_spaces_between_tags(value):
-    """Returns the given HTML with spaces between tags removed."""
+    """Return the given HTML with spaces between tags removed."""
     return re.sub(r'>\s+<', '><', force_text(value))
-strip_spaces_between_tags = allow_lazy(strip_spaces_between_tags, six.text_type)
-
-
-def strip_entities(value):
-    """Returns the given HTML with all entities (&something;) stripped."""
-    warnings.warn(
-        "django.utils.html.strip_entities() is deprecated.",
-        RemovedInDjango110Warning, stacklevel=2
-    )
-    return re.sub(r'&(?:\w+|#\d+);', '', force_text(value))
-strip_entities = allow_lazy(strip_entities, six.text_type)
 
 
 def smart_urlquote(url):
-    "Quotes a URL if it isn't already quoted."
+    """Quote a URL if it isn't already quoted."""
     def unquote_quote(segment):
-        segment = unquote(force_str(segment))
+        segment = unquote(segment)
         # Tilde is part of RFC3986 Unreserved Characters
         # http://tools.ietf.org/html/rfc3986#section-2.3
         # See also http://bugs.python.org/issue16285
-        segment = quote(segment, safe=RFC3986_SUBDELIMS + RFC3986_GENDELIMS + str('~'))
+        segment = quote(segment, safe=RFC3986_SUBDELIMS + RFC3986_GENDELIMS + '~')
         return force_text(segment)
 
     # Handle IDN before quoting.
@@ -242,7 +206,7 @@ def smart_urlquote(url):
     if query:
         # Separately unquoting key/value, so as to not mix querystring separators
         # included in query values. See #22267.
-        query_parts = [(unquote(force_str(q[0])), unquote(force_str(q[1])))
+        query_parts = [(unquote(q[0]), unquote(q[1]))
                        for q in parse_qsl(query, keep_blank_values=True)]
         # urlencode will take care of quoting
         query = urlencode(query_parts)
@@ -253,22 +217,22 @@ def smart_urlquote(url):
     return urlunsplit((scheme, netloc, path, query, fragment))
 
 
+@keep_lazy_text
 def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
     """
-    Converts any URLs in text into clickable links.
+    Convert any URLs in text into clickable links.
 
     Works on http://, https://, www. links, and also on links ending in one of
     the original seven gTLDs (.com, .edu, .gov, .int, .mil, .net, and .org).
     Links can have trailing punctuation (periods, commas, close-parens) and
     leading punctuation (opening parens) and it'll still do the right thing.
 
-    If trim_url_limit is not None, the URLs in the link text longer than this
-    limit will be truncated to trim_url_limit-3 characters and appended with
-    an ellipsis.
+    If trim_url_limit is not None, truncate the URLs in the link text longer
+    than this limit to trim_url_limit-3 characters and append an ellipsis.
 
-    If nofollow is True, the links will get a rel="nofollow" attribute.
+    If nofollow is True, give the links a rel="nofollow" attribute.
 
-    If autoescape is True, the link text and URLs will be autoescaped.
+    If autoescape is True, autoescape the link text and URLs.
     """
     safe_input = isinstance(text, SafeData)
 
@@ -279,8 +243,8 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
 
     def unescape(text, trail):
         """
-        If input URL is HTML-escaped, unescape it so as we can safely feed it to
-        smart_urlquote. For example:
+        If input URL is HTML-escaped, unescape it so that it can be safely fed
+        to smart_urlquote. For example:
         http://example.com?x=1&amp;y=&lt;2&gt; => http://example.com?x=1&y=<2>
         """
         unescaped = (text + trail).replace(
@@ -295,24 +259,61 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
             trail = ''
         return text, unescaped, trail
 
-    words = word_split_re.split(force_text(text))
-    for i, word in enumerate(words):
-        if '.' in word or '@' in word or ':' in word:
-            # Deal with punctuation.
-            lead, middle, trail = '', word, ''
-            for punctuation in TRAILING_PUNCTUATION:
-                if middle.endswith(punctuation):
-                    middle = middle[:-len(punctuation)]
-                    trail = punctuation + trail
+    def trim_punctuation(lead, middle, trail):
+        """
+        Trim trailing and wrapping punctuation from `middle`. Return the items
+        of the new state.
+        """
+        # Continue trimming until middle remains unchanged.
+        trimmed_something = True
+        while trimmed_something:
+            trimmed_something = False
+
+            # Trim trailing punctuation.
+            stripped = middle.rstrip(TRAILING_PUNCTUATION_CHARS)
+            if middle != stripped:
+                trail = middle[len(stripped):] + trail
+                middle = stripped
+                trimmed_something = True
+
+            # Trim wrapping punctuation.
             for opening, closing in WRAPPING_PUNCTUATION:
                 if middle.startswith(opening):
                     middle = middle[len(opening):]
-                    lead = lead + opening
+                    lead += opening
+                    trimmed_something = True
                 # Keep parentheses at the end only if they're balanced.
-                if (middle.endswith(closing)
-                        and middle.count(closing) == middle.count(opening) + 1):
+                if (middle.endswith(closing) and
+                        middle.count(closing) == middle.count(opening) + 1):
                     middle = middle[:-len(closing)]
                     trail = closing + trail
+                    trimmed_something = True
+        return lead, middle, trail
+
+    def is_email_simple(value):
+        """Return True if value looks like an email address."""
+        # An @ must be in the middle of the value.
+        if '@' not in value or value.startswith('@') or value.endswith('@'):
+            return False
+        try:
+            p1, p2 = value.split('@')
+        except ValueError:
+            # value contains more than one @.
+            return False
+        # Dot must be in p2 (e.g. example.com)
+        if '.' not in p2 or p2.startswith('.'):
+            return False
+        return True
+
+    words = word_split_re.split(force_text(text))
+    for i, word in enumerate(words):
+        if '.' in word or '@' in word or ':' in word:
+            # lead: Current punctuation trimmed from the beginning of the word.
+            # middle: Current state of the word.
+            # trail: Current punctuation trimmed from the end of the word.
+            lead, middle, trail = '', word, ''
+            # Deal with punctuation.
+            lead, middle, trail = trim_punctuation(lead, middle, trail)
 
             # Make URL we want to point to.
             url = None
@@ -323,7 +324,7 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
             elif simple_url_2_re.match(middle):
                 middle, middle_unescaped, trail = unescape(middle, trail)
                 url = smart_urlquote('http://%s' % middle_unescaped)
-            elif ':' not in middle and simple_email_re.match(middle):
+            elif ':' not in middle and is_email_simple(middle):
                 local, domain = middle.rsplit('@', 1)
                 try:
                     domain = domain.encode('idna').decode('ascii')
@@ -350,7 +351,6 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
         elif autoescape:
             words[i] = escape(word)
     return ''.join(words)
-urlize = allow_lazy(urlize, six.text_type)
 
 
 def avoid_wrapping(value):
@@ -371,22 +371,12 @@ def html_safe(klass):
             "can't apply @html_safe to %s because it defines "
             "__html__()." % klass.__name__
         )
-    if six.PY2:
-        if '__unicode__' not in klass.__dict__:
-            raise ValueError(
-                "can't apply @html_safe to %s because it doesn't "
-                "define __unicode__()." % klass.__name__
-            )
-        klass_unicode = klass.__unicode__
-        klass.__unicode__ = lambda self: mark_safe(klass_unicode(self))
-        klass.__html__ = lambda self: unicode(self)  # NOQA: unicode undefined on PY3
-    else:
-        if '__str__' not in klass.__dict__:
-            raise ValueError(
-                "can't apply @html_safe to %s because it doesn't "
-                "define __str__()." % klass.__name__
-            )
-        klass_str = klass.__str__
-        klass.__str__ = lambda self: mark_safe(klass_str(self))
-        klass.__html__ = lambda self: str(self)
+    if '__str__' not in klass.__dict__:
+        raise ValueError(
+            "can't apply @html_safe to %s because it doesn't "
+            "define __str__()." % klass.__name__
+        )
+    klass_str = klass.__str__
+    klass.__str__ = lambda self: mark_safe(klass_str(self))
+    klass.__html__ = lambda self: str(self)
     return klass
